
/* 
 * OverAll.m -- Implementation file for the OverAll class 
 *
 * You may freely copy, distribute, and reuse the code in this example.
 * NeXT disclaims any warranty of any kind, expressed or implied, as to 
 * its fitness for any particular use, nor even that it has a use.
 *
 */

#import "OverAll.h"
#import "nxlcau24.h"
#import "rec.h"
#import <math.h> 
#import <dpsclient/psops.h>
#import <dpsclient/wraps.h>
#import <stdlib.h>
#import <string.h>
#import <time.h>

extern int    co, dn;			/* connectivity, dimension */
       int    tc[DS];
extern double bx[BD];			/* bernstein factor        */
extern double lf[BL];			/* Life 2d gen Bern coeffs */

char dfclr[KK][12] = {"1.0,0.0,0.0","0.0,1.0,0.0"};

NXDefaultsVector defaults = { {"well0", "1.0, 0.0, 0.0"},
			      {"well1", "0.0, 1.0, 0.0"},
			      {NULL, NULL} };

char    colorwells[KK][6]={"well0", "well1" };
NXColor colst[KK];

extern struct fptbl dtbl[];
extern Inst xprg[XLEN];
extern char idef[][30];


@implementation OverAll

/* ---------------  A P P   I N I T I A L I Z A T I O N   ----------------- */

- appWillInit:sender {printf("appWillInit\n"); return self;}

- appDidInit:sender  {
char c[20], ch[WNML], *df ;int i,o; double red, green, blue;
struct tm  *timeptr;
time_t  secsnow;
  printf("appDidInit = nxlcau24\n");
  co=1;		/* normal connectivity */
  dn=2;		/* two-dimensional     */
  dn=1;		/* one-dimensional     */
//  rule24();     /* printf("rule24\n"); */
//  ttzero();
//  stzero();
//  gfzero();
  if (dn==1) {
    ranblin(0.5,0.5);
    grantotrule();
    totoasc();
    tazero();
    asctobin();
    [rulePanel setAllRules: self];
    }
//  [rulePanel setAllRules: self];
//  [self showProbability: pSliders];
  if (dn==2) {
    liferule();
    conlif();
    golta();
    ranplacor();
    asctobin();
  }

for (i=0; i<BD; i++) bx[i]=1.0;				/* bernstein factor */

lf[ 0]=      0.0;
lf[ 1]=      0.0;
lf[ 2]=      0.0;
lf[ 3]=     22.0;
lf[ 4]=   1092.0;
lf[ 5]=  10902.0;
lf[ 6]=  52808.0;
lf[ 7]= 159532.0;
lf[ 8]= 352308.0;
lf[ 9]= 662834.0;
lf[10]=1136852.0;
lf[11]=1653846.0;
lf[12]=1844296.0;
lf[13]=1487120.0;
lf[14]= 813480.0;
lf[15]= 273548.0;
lf[16]=  49756.0;
lf[17]=   3962.0;
lf[18]=     72.0;
lf[19]=      0.0;
lf[20]=      0.0;
lf[21]=      0.0;
lf[22]=      0.0;
lf[23]=      0.0;
lf[24]=      0.0;
lf[25]=      0.0;

   golta();				/* do always, as precaution */
   totta();				/* do always, as precaution */

   time(&secsnow);
   timeptr = localtime(&secsnow);
   sprintf(c,"%d-%d-19%02d\n",
	((timeptr->tm_mon) + 1), 
	timeptr->tm_mday,
        timeptr->tm_year);
   [tiDat setStringValue: c];

   NXRegisterDefaults("NXLCAU24Defaults", defaults);
   NXUpdateDefaults();
    for(i=0; i <KK; i++) {
     df=NXGetDefaultValue("NXLCAU24Defaults",colorwells[i]);
     if (df==NULL) {
	NXWriteDefault("NXLCAU24Defaults",colorwells[i],dfclr[i]);
	df=&dfclr[i];
	}
     sscanf(df,"%6lf,%6lf,%6lf",&red,&green,&blue); /* don't use %F */
     colst[i]=NXConvertRGBToColor(red,green,blue);
     printf("%s: %s\n",colorwells[i],df);
     }

  [[NXColorPanel sharedInstance: YES] setContinuous: YES];
  [self updateColors];

   /*  initialize r_vst with selected standard procedures */
   rec_c(idef[0], xprg[0], XLEN, dtbl);
   r_vst['a']=&xprg[0];

   [browser getTitleFromPreviousColumn: YES];
   [browser setDoubleAction: @selector(copyAndRunREC:)];
   [browser setPath: "/demonstrations"];

   /*  initialize relations panel  */
//   [self loadLowerPanel:arithmRuleBox];
//   [self loadBox:self];
return self;
}
 
- appWillTerminate:sender {printf("appWillTerminate\n"); return self;}

/* generate the totalistic code table */
void totta() {int i0, i1, i2, i3, i4, i5, i6, i7, i8;
  for (i8=0; i8<KK; i8++)
  for (i7=0; i7<KK; i7++)
  for (i6=0; i6<KK; i6++)
  for (i5=0; i5<KK; i5++)
  for (i4=0; i4<KK; i4++)
  for (i3=0; i3<KK; i3++)
  for (i2=0; i2<KK; i2++)
  for (i1=0; i1<KK; i1++)
  for (i0=0; i0<KK; i0++)
    tc[i0+KK*(i1+KK*(i2+KK*(i3+KK*(i4+KK*(i5+KK*(i6+KK*(i7+KK*i8)))))))] = 
	i0+i1+i2+i3+i4+i5+i6+i7+i8;
}

/* ----------------------------- G E N E R A L ----------------------------- */

- saveColors:sender {char c[32]; int i;
    colst[0]=[inCol0 color];
    colst[1]=[inCol1 color];
  for (i=0; i<KK; i++) {
        sprintf(c,"%6.3f, %6.3f, %6.3f",
		NXRedComponent(colst[i]),
		NXGreenComponent(colst[i]),
		NXBlueComponent(colst[i]));
	NXWriteDefault("NXLCAU24Defaults",colorwells[i],c);
	};
return self;
}

- revertColors:sender {char *df; int i; double red, green, blue; 
  for (i=0; i<KK; i++) {
    df=NXGetDefaultValue("NXLCAU24Defaults",colorwells[i]);
       if (df==NULL) {printf("null df = %s\n",df); return self;}
       sscanf(df,"%6lf,%6lf,%6lf",&red,&green,&blue); 
    colst[i]=NXConvertRGBToColor(red,green,blue);
    printf("%s: %s\n",colorwells[i],df);
    }
  [self updateColors];
return self;
}

- updateColors {
  [inCol0 setColor: colst[0]];
  [inCol1 setColor: colst[1]];
return self;
}

- windowDidBecomeKey:sender {
	[menuCopy setTarget: self];
	[menuCopy setAction: @selector(copyWells:)];
	[menuCopy setEnabled: YES];
return self;
}

- windowDidResignKey:sender {[menuCopy setEnabled: NO]; return self;}

- copyWells:sender {
NXRect zbounds;
id     pBoard;
  pBoard = [Pasteboard new];
  [pBoard declareTypes: &NXPostScriptPboardType num: 1 owner: self];
  [theWellBox getBounds: &zbounds];
  [theWellBox writePSCodeInside: &zbounds to: pBoard]; 
return self;
}

/* - - - - - - - -  R E C   L I N E   C O N F I G U R A T I O N  - - - - - */
 
- useRECLine:sender {
  [RECPanel makeKeyAndOrderFront: sender];
  [RECValue setStringValue: "*"];
 return self;
}

- runRECLine:sender {
  [RECValue setStringValue: " "];
  if (rec([sender stringValue])) {
     [RECValue setStringValue: "T"];
     [rulePanel setAllRules: self];
     }
  else
    [RECValue setStringValue: "F"];
  return self;
}

/* << S E T - D I M E N S I O N - A N D - L O A D - A C C O R D I N G L Y >> */

- loadOverAll:sender {
   if (ardnPanel==nil) {
      if (![NXApp loadNibSection: "overall.nib" owner: self withNames: NO]) 
	{return nil;}
    }
  if ([sender selectedCell]==menuArdn) 
	[ardnPanel makeKeyAndOrderFront: sender];
  if ([sender selectedCell]==menuColor) 
	[colorPanel makeKeyAndOrderFront: sender];
  if ([sender selectedCell]==menuPair)  
	[pairPanel makeKeyAndOrderFront: sender];
  if ([sender selectedCell]==menuSset)  
	[ssetPanel makeKeyAndOrderFront: sender];
  return self;
}

- dimenChoice:sender {
   if (dimenPrefs==nil) {
      if (![NXApp loadNibSection: "overall.nib" owner: self withNames: NO]) 
	{return nil;}
      [dimenMenu setEmptySelectionEnabled: YES];
      [dimenMenu selectCellAt: -1: 0];
    }
  [dimenPrefs makeKeyAndOrderFront: sender];
  return self;
}

- setDimension:sender {dn = [sender selectedRow]+1; return self;}

- dbWindow:sender {
   switch (dn) {
     case 1: [rijnView loadRijn: sender]; break;
     case 2: [ruinView showRuin: sender]; break;
     default: break;}
printf("a de BruinWindow was loaded\n");
  return self;
}

@end

/* end OverAll.m */
