
/* 
 * SQULView.m -- Implementation file for the SQULView class 
 *
 * You may freely copy, distribute, and reuse the code in this example.
 * NeXT disclaims any warranty of any kind, expressed or implied, as to 
 * its fitness for any particular use, nor even that it has a use.
 *
 */

#import "SQULView.h"
#import "drawingFuncs.h"
#import "nxlcau24.h"
#import <math.h> 
#import <stdlib.h>
#import <string.h>

#define MB 12			/* maximum binomial coefficient */
#define NG 80			/* number of Golay Classes	*/
#define GH    (DS/KK)		/* DS/KK= half number of neighborhoods	*/

extern char ascrule[KK][KK][KK][KK][KK][KK][KK][KK][KK];
extern char auxrule[KK][KK][KK][KK][KK][KK][KK][KK][KK];
extern char golRule[];			/* evolutionary rule    */
extern char ttrule[];			/* evolutionary rule    */
char   ulrule[DS];			/* evolutionary rule    */
char   uarule[DS];			/* auxiliary rule flags */

extern char    arr1[], arr2[];
       int             brr2[AL];
extern double  bp[];			/* Bernstein coefficients    */
extern int     tc[DS];
       int     ulgus[2*NG];		/* golay use statistics    */
       int     ulrus[DS];		/* rule use statistics     */
       int     ultus[TS];		/* totalistic use stats    */
       int     ullf[KK];		/* number of live cells	     */
       int     ulllf[KK];		/* save number of live cells */
int     nr[DS];		/* nbrhd representative of symm class 	*/
int     rs[DS];		/* rule use statistics 			*/
int     gc[NG];		/* Golay code key 			*/
int	sc[NG][BD];
int     wc[NG];		/* size (weight) of symm class 		*/

int     totb[DS];
int     gotb[DS];

extern NXColor colst[KK];    		/* Array for state colors    */

int    cellULNood=NO;
int    checkULBoard=NO;
int    drawULBack=YES;
int    drawULBnstein=NO;
int    drawULFreqs=NO;
int    drawULQuency=NO;
int    drawULRturn=NO;
int    drawULSmall=NO;
int    drawULTrajec=NO;
int    slowULFreqs=NO;
int    smallULSmall=NO;
int    smallULMosaic=NO;

int    lastULPlane=0;			/* 0 = none, 1 = evol, 2 = mosaic */
int    lastULTraj=0;			/* 0 = none, 1 = traj, 2 = return */
int    moULTiles=3;
int    pQULDots=NO;
int    trajULLen=10;
int    repULPlEv;
int    rturnULNum=1;
int    skipULThree=NO;
int    sttretch=125;

int     tottpmut[TS] = {0,1,2,3,4,5,6,7,8,9};
NXColor tottcolst[TS];    		/* Array for totalistic colors   */
int     ULgtb[NG] = {0,1,2,2,3,4};

extern int    drawview;
extern int    co, dn;

void    argfram(int n, int l);
void    argdiag(int n, int l);
void    ulonegen(char *aa, int *bb, int m, int n); 
void    asctobin();
void	asctoqul();
void	asczero();
void    bargraf(int m, int n);
void    ulbdot(int i, int j, int l);
void    ulbline(double x, double y, int l);
int     istot();
int     ix2(int i, int j);
int     ix3(int i, int j, int k);
int     nearest(double x, double y);
void    onegen(int n);
void    ranblin(double p1, double p2);
void    renewarry();
void    savearry();
void    screen(char *aa, int m, int n);
void    ulstatecount(char *aa, int m, int n);
void    suggULrule(int i);
void    videodot(int i, int j, int l);
void    wqusscrrul(char *c, char ru[KK][KK][KK][KK][KK][KK][KK][KK][KK]);
void    wrru(char *c, char ru[KK][KK][KK][KK][KK][KK][KK][KK][KK]);
void    zerline();

@implementation SQULView
 
/* -------------------------------- S Q U L -------------------------------- */

/* put up the principal window for the PairView. Since pair.nib is a 	  */
/* secondary .nib file, it is not loaded until there is a demand, which   */
/* would usually arise from the "Pair" option in the main menu.	   	  */

- loadSqul:sender {
  if (squlWindow==nil) {
    if (![NXApp loadNibSection: "squl.nib" owner: self withNames: NO]) 
	{return nil;}
    }
  dn=2;
  [SqulWindow makeKeyAndOrderFront: sender];
  [squlWindow orderFront: nil];
  [self copyRule: sender];
  if (istot()) [self copyTRule: sender];
return self;
}

- printSQUL:sender {
   if (keyWindow==SqulWindow) 
	{[SqulWindow smartPrintPSCode: self];}
   else if (keyWindow==planeWindow) {
    switch (lastULPlane) {
      case 0: break;
      case 3: drawULSmall=YES;   break;
      case 4: smallULMosaic=YES; break;
      default: break;}
    [planeWindow smartPrintPSCode: self];
    switch (lastULPlane) {
      case 0: break;
      case 3: drawULSmall=NO;   break;
      case 4: smallULMosaic=NO; break;
      default: break;}
    }
return self;
}

- windowDidBecomeKey:sender {
	keyWindow=sender;
	[menuCopy setTarget: self];
	[menuCopy setAction: @selector(copySQUL:)];
	[menuCopy setEnabled: YES];
	[menuPrint setTarget: self];
	[menuPrint setAction: @selector(printSQUL:)];
	[menuPrint setEnabled: YES];
return self;
}

- windowDidResignKey:sender {
	[menuCopy setEnabled: NO];
	[menuPrint setEnabled: NO];
return self;
}

- copySqulView:sender {
NXRect zbounds;
id     pBoard;
  pBoard = [Pasteboard new];
  [pBoard declareTypes: &NXPostScriptPboardType num: 1 owner: self];
  [SqulView getBounds: &zbounds];
  [SqulView writePSCodeInside: &zbounds to: pBoard]; 
return self;
}

- copyWhoosit: (int *) flag {
NXRect zbounds;
id     pBoard;
  *flag = YES;
  drawview=13;
  drawULBack=NO;
  pBoard = [Pasteboard new];
  [pBoard declareTypes: &NXPostScriptPboardType num: 1 owner: self];
  [trajView getBounds: &zbounds];
  [trajView writePSCodeInside: &zbounds to: pBoard]; 
  drawULBack=YES;
  *flag = NO;
return self;
}

- copyWhatsit: (int *) flag {
NXRect zbounds;
id     pBoard;
  *flag = YES;
  drawULBack=NO;
  pBoard = [Pasteboard new];
  [pBoard declareTypes: &NXPostScriptPboardType num: 1 owner: self];
  [planeView getBounds: &zbounds];
  [planeView writePSCodeInside: &zbounds to: pBoard]; 
  drawULBack=YES;
  *flag = NO;
return self;
}

- copySQUL:sender {
   savearry();
   if (keyWindow==SqulWindow) {[self copySqulView: self];}
   else if (keyWindow==trajWindow) switch (lastULTraj) {
          case 0: break;
          case 1: [self copyWhoosit: &drawULTrajec]; break;
          case 2: [self copyWhoosit: &drawULRturn];  break;
          case 3: [self copyWhoosit: &drawULFreqs];  break;
          case 4: [self copyWhoosit: &slowULFreqs];  break;
          default: break;}
   else if (keyWindow==planeWindow) switch (lastULPlane) {
        case 0: break;
        case 3: drawview=7;  [self copyWhatsit: &drawULSmall];   break;
        case 4: drawview=11; [self copyWhatsit: &smallULMosaic]; break;
        default: break;}
   renewarry();
return self;
}

- showProbability:sender {int i; double pp, p[KK];
  for (i=0; i<KK; i++) p[i]=[[sender cellAt: i: 0] doubleValue];
  for (pp=0, i=0; i<KK; i++) pp+=p[i];
  for (i=0; i<KK; i++) {
    [[pSliders cellAt: i: 0] setDoubleValue: p[i]/pp];
    [[pWindows cellAt: i: 0] setDoubleValue: p[i]/pp];
    }
return self;
}

- randomLine:sender {double p[KK]; int i;
  for (i=0; i<KK; i++) p[i]=[[pSliders cellAt:i:0] doubleValue];
  ranblin(p[0],p[1]);
return self;
}

- sCellLine:sender {
  smallULSmall=YES;
  [planeView display];
  smallULSmall=NO;
return self;
}

- uniformLine:sender {ranblin(0.5,0.5); return self;}

- checkerBoard:sender {
  checkULBoard=YES;
  [planeView display];
  checkULBoard=NO;
return self;
}

- uprule:sender {suggULrule(0); [self anyrule: self]; return self;}
- lerule:sender {suggULrule(1); [self anyrule: self]; return self;}
- dorule:sender {suggULrule(3); [self anyrule: self]; return self;}
- rirule:sender {suggULrule(2); [self anyrule: self]; return self;}
- idrule:sender {suggULrule(4); [self anyrule: self]; return self;}
- mjrule:sender {suggULrule(5); [self anyrule: self]; return self;}

- anyrule:sender {
   [self copyRule: self];
   [rulePanel setAllRules: self];
return self;}

void suggULrule(i) int i; {int j, i0, i1, i2, i3, i4, i5, i6, i7, i8;
  for (i0=0; i0<KK; i0++)
  for (i1=0; i1<KK; i1++)
  for (i2=0; i2<KK; i2++)
  for (i3=0; i3<KK; i3++)
  for (i4=0; i4<KK; i4++)
  for (i5=0; i5<KK; i5++)
  for (i6=0; i6<KK; i6++)
  for (i7=0; i7<KK; i7++)
  for (i8=0; i8<KK; i8++) {
    switch (i) {
      case 0: j=i2; break;
      case 1: j=i3; break;
      case 2: j=i1; break;
      case 3: j=i0; break;
      case 4: j=i4; break;
      case 5: j=0; if (i0+i1+i2+i3+i4+i5+i6+i7+i8>4) j=1; break;
      default: j=0; break;}
    ascrule[i0][i1][i2][i3][i4][i5][i6][i7][i8] = '0'+j;
    }
}

/* --------------------------- E V O L U T I O N --------------------------- */

- drawSmEvol:sender {int k;
    drawview=7;
    drawULSmall=YES;
    asctoqul();
    if ([reStore state]) savearry();
    sscanf([[sender selectedCell] title],"%d",&repULPlEv);
    [self display];
    if (repULPlEv>0) for (k=0; k<repULPlEv; k++) [self display];
    if ([reStore state]) renewarry();
    drawULSmall=NO;
    lastULPlane=3;
return self;
}

- printSmEvol:sender {
    drawview=7;
    drawULSmall=YES;
    repULPlEv=0;
    [keyWindow smartPrintPSCode: self];
    drawULSmall=NO;
return self;
}

- drawSmMosaic:sender {
    cellULNood=[cellNhood state];
    moULTiles=[moTies selectedRow]+2;
    drawview=11;
    smallULMosaic=YES;
    asctoqul();
    savearry();
    [self display];
    renewarry();
    smallULMosaic=NO;
    lastULPlane=4;
return self;
}

- printSmMosaic:sender {
    drawview=11;
    smallULMosaic=YES;
    savearry();
    [keyWindow smartPrintPSCode: self];
    renewarry();
    smallULMosaic=NO;
return self;
}

- drawTraj:sender {
    sscanf([[sender selectedCell] title],"%d",&trajULLen);
    pQULDots=[probDots state];
    drawview=13;
    drawULTrajec=YES;
    asctoqul();
    savearry();
    [trajView display];
    renewarry();
    drawULTrajec=NO;
    lastULTraj=1;
return self;
}

- printTraj:sender {
    drawview=13;
    drawULTrajec=YES;
    savearry();
    [keyWindow smartPrintPSCode: self];
    renewarry();
    drawULTrajec=NO;
return self;
}

- drawRtrn:sender {
    sscanf([[sender selectedCell] title],"%d",&rturnULNum);
    drawview=13;
    drawULRturn=YES;
    asctoqul();
    savearry();
    [trajView display];
    renewarry();
    drawULRturn=NO;
    lastULTraj=2;
return self;
}

- printRtrn:sender {
    drawview=13;
    drawULRturn=YES;
    savearry();
    [keyWindow smartPrintPSCode: self];
    renewarry();
    drawULRturn=NO;
return self;
}

- drawFreq:sender {
//    sscanf([[sender selectedCell] title],"%d",&rturnULNum);
    drawview=13;
    drawULFreqs=YES;
    asctoqul();
    savearry();
    [trajView display];
    renewarry();
    drawULFreqs=NO;
    lastULTraj=3;
return self;
}

- printFreq:sender {
    drawview=13;
    drawULFreqs=YES;
    savearry();
    [keyWindow smartPrintPSCode: self];
    renewarry();
    drawULFreqs=NO;
return self;
}

- slowFreq:sender {
//    sscanf([[sender selectedCell] title],"%d",&rturnULNum);
    drawview=13;
    slowULFreqs=YES;
    asctoqul();
    savearry();
    [trajView display];
    renewarry();
    slowULFreqs=NO;
    lastULTraj=4;
return self;
}

- printSlowFreq:sender {
    drawview=13;
    slowULFreqs=YES;
    savearry();
    [keyWindow smartPrintPSCode: self];
    renewarry();
    slowULFreqs=NO;
return self;
}

- drawFrequency:sender {
printf("drawing Frequency\n");
    skipULThree=[eachAndAll state];
    tottpmut[0]=[freqPerm0 selectedCol];
    tottpmut[1]=[freqPerm1 selectedCol];
    tottpmut[2]=[freqPerm2 selectedCol];
    tottpmut[3]=[freqPerm3 selectedCol];
    tottpmut[4]=[freqPerm4 selectedCol];
    tottpmut[5]=[freqPerm5 selectedCol];
    tottpmut[6]=[freqPerm6 selectedCol];
    tottpmut[7]=[freqPerm7 selectedCol];
    tottpmut[8]=[freqPerm8 selectedCol];
    tottpmut[9]=[freqPerm9 selectedCol];
    tottcolst[0]=[freqColor0 color];
    tottcolst[1]=[freqColor1 color];
    tottcolst[2]=[freqColor2 color];
    tottcolst[3]=[freqColor3 color];
    tottcolst[4]=[freqColor4 color];
    tottcolst[5]=[freqColor5 color];
    tottcolst[6]=[freqColor6 color];
    tottcolst[7]=[freqColor7 color];
    tottcolst[8]=[freqColor8 color];
    tottcolst[9]=[freqColor9 color];
    drawview=14;
    asctoqar();
    if ([reStock state]) savearry();
    if ([stretchOut state]) {
	[stretchWindow makeKeyAndOrderFront: sender];
        drawULQuency=YES;
	sttretch=375;
	[stretchView display];
	sttretch=125;
	} else {drawULQuency=YES; [freqView display];}
    if ([reStock state]) renewarry();
    drawULQuency=NO;
return self;
}

- printFrequency:sender {
    drawview=14;
    drawULQuency=YES;
    savearry();
    [keyWindow smartPrintPSCode: self];
    renewarry();
    drawULQuency=NO;
return self;
}

/* draw rectangle (i,j) in a mosaic of (u,v) */
/* whose individual rectangles measure (m,n) */
void manyULrect(i,j,u,v,m,n) int i, j, u, v, m, n; {int k, l, pj;
  asctoqul();
  for (k=0; k<m; k++) for (l=0; l<n; l++) 
    if (cellULNood) {
		 pj=tottpmut[tc[brr2[k+n*l]]];
		 NXSetColor(tottcolst[pj]);
	         videodot(i*m+k,j*n+l,5);
		 }
    else videodot(i*m+k,j*n+l,arr1[k*n+l]);
  ulonegen(arr1,brr2,m,n);
}

/* m = high/moULTiles, n = wide/moULTiles */
void smallULgrid(m,n) int m, n; {int i; double em, en, mo;
  em=(double)m;
  en=(double)n;
  mo=(double)moULTiles;
	  PSsetgray(NX_BLACK); 
	  PSsetlinewidth(0.1);
  for (i=0; i<moULTiles+1; i++) {
  PSmoveto((double)(en*i),0.0); PSlineto((double)(en*i),mo*em);}
  for (i=0; i<moULTiles+1; i++) {
  PSmoveto(0.0,(double)(em*i)); PSlineto(mo*en,(double)(em*i));}
  PSstroke();
}

/* -----------------------  D E F I N E   R U L E S  ----------------------- */

/* - - - - - - - -   B E R N S T E I N   H I S T O G R A M S   - - - - - - */

- drawBern:sender {
    drawview=12;
    drawULBnstein=YES;
    [self display];
    drawULBnstein=NO;
    [window setDocEdited: YES];
return self;
}

- selfBern:sender {
    drawview=12;
    drawULBnstein=YES;
    [self display];
    drawULBnstein=NO;
    [window setDocEdited: YES];
return self;
}

- printRule:sender {
   drawview=12;
   drawULBnstein=YES;
   [keyWindow printPSCode:self];
   drawULBnstein=NO;
return self;
}

- readRule:sender 
    {[rulePanel readWRule: sender]; [self copyRule: self]; return self;}

- enterRule:sender {char ch[WNML];
char    tiempo[30], fecha[30];
struct  tm  *timeptr;
time_t  secsnow;

   time(&secsnow);
   timeptr = localtime(&secsnow);
   sprintf(tiempo,"%02d:%02d:%02d\n",
	timeptr->tm_hour, 
	timeptr->tm_min,
        timeptr->tm_sec);
   [tiTime setStringValue: tiempo];
   sprintf(fecha,"%d-%d-19%02d\n",
	((timeptr->tm_mon) + 1), 
	timeptr->tm_mday,
        timeptr->tm_year);
   [tiDat setStringValue: fecha];

      asczero(ascrule);
///***///
     wrru(ch,ascrule);
     [ruleNumber setStringValue: ch];
     [rulePanel setAllRules: self];
     [SqulWindow setDocEdited: NO];
     if (istot()) [ruleType setStringValue:"totalistic"];
     else [ruleType setStringValue:""];
return self;
}

- resetRule:sender {int i, j; 
    for (i=0; i<NG; i++) for (j=0; j<MB; j++) {
	[[theOMatrix cellAt: j: i] setIntValue: NO];
	[[theZMatrix cellAt: j: i] setIntValue: NO];
	}
return self;
}

- copyRule:sender {char ch[WNML]; int i;
///***///
     wrru(ch,ascrule);
     [ruleNumber setStringValue: ch];
     wqusscrrul(ch,ascrule);
     [xsRuleNumber setStringValue: ch];
     if (istot()) {
	[tRuleNumber setIntValue: totonu(0)];
	for (i=0; i<TS; i++) 
	  [[theToMatrix cellAt: 0: i] setIntValue: ttrule[i]-'0'];
	}
     else [tRuleNumber setStringValue: "enter totalistic #"];
     [SqulWindow setDocEdited: NO];
     [bernView drawBern: nil];
return self;
}

- readTRule:sender 
    {[rulePanel readTRule: sender]; [self copyRule: self]; return self;}

- enterTRule:sender {char ch[WNML];
char    tiempo[30], fecha[30];
struct  tm  *timeptr;
time_t  secsnow;

   time(&secsnow);
   timeptr = localtime(&secsnow);
   sprintf(tiempo,"%02d:%02d:%02d\n",
	timeptr->tm_hour, 
	timeptr->tm_min,
        timeptr->tm_sec);
   [tiTime setStringValue: tiempo];
   sprintf(fecha,"%d-%d-19%02d\n",
	((timeptr->tm_mon) + 1), 
	timeptr->tm_mday,
        timeptr->tm_year);
   [tiDat setStringValue: fecha];

     ttrule[[sender selectedCol]] = [[sender selectedCell] state]+'0';
     totoasc();
     wrru(ch,ascrule);
     [ruleNumber setStringValue: ch];
     [rulePanel setAllRules: self];
     [SqulWindow setDocEdited: NO];
     if (istot()) [tRuleNumber setIntValue: totonu(0)];
       else [tRuleNumber setStringValue: "enter totalistic #"];
     [bernView drawBern: self];
return self;
}

- copyTRule:sender {char ch[WNML]; int i;
     wrru(ch,ascrule);
     [ruleNumber setStringValue: ch];
     if (istot()) {
		[tRuleNumber setIntValue: totonu(0)];
		for (i=0; i<TS; i++) 
		  [[theToMatrix cellAt: 0: i] setIntValue: ttrule[i]-'0'];
		}
       else [tRuleNumber setStringValue: "enter totalistic #"];
     [SqulWindow setDocEdited: NO];
     [squlView selfBern: nil];
return self;
}

void berncoeff() {int i, i0, i1, i2, i3, i4, i5, i6, i7, i8;
  for (i=0; i<BD; i++) bp[i]=0.0;
  for (i0=0; i0<KK; i0++)
  for (i1=0; i1<KK; i1++)
  for (i2=0; i2<KK; i2++)
  for (i3=0; i3<KK; i3++)
  for (i4=0; i4<KK; i4++)
  for (i5=0; i5<KK; i5++)
  for (i6=0; i6<KK; i6++)
  for (i7=0; i7<KK; i7++)
  for (i8=0; i8<KK; i8++)
    if (ascrule[i0][i1][i2][i3][i4][i5][i6][i7][i8]=='1')
      bp[i0+i1+i2+i3+i4+i5+i6+i7+i8]+=1.0;
}

/* -------------------------------- * * * * -------------------------------- */

- initFrame:(const NXRect *)frameRect
/*
 * Initializes the new SQULView object.  First, an initFrame: message is sent 
 * to super to initialize SQULView as a View.  Next, the SQULView sets its own 
 * state -- that it is opaque and that the origin of its coordinate system lies 
 * in the center of its area.  It then creates and initializes its associated 
 * objects, a Storage object, an NXCursor, and a Cell.  Finally, it loads into 
 * the Window Server some PostScript procedures that it will use in drawing 
 * itself. 
 */
{
    NXPoint	spot;
	
    [super initFrame:frameRect];
/*  [self translate:floor(frame.size.width/2) :floor(frame.size.height/2)]; */
    [self setOpaque:YES];
    crossCursor=[NXCursor newFromImage:[NXImage newFromSection:"cross.tiff"]];
    spot.x = spot.y = 7.0;
    [crossCursor setHotSpot:&spot];

  PSsetgray(NX_BLACK);
  PSselectfont("Times-Roman",16.0);

  [self setFlipped: YES];
  drawview=7;			      /* prefer origin in upper left corner */
	return self;
}

- setDelegate:anObject
/*
 * Sets the SQULView's delegate instance variable to the supplied object.
 */
{
	delegate = anObject;
	return self;
}

- drawSelf:(const NXRect *)rects :(int)rectCount
/*
 * Draws the SQULView's background and axes.  If there are any points,
 * these are drawn too.
 *
 * (Note:  For simplicity, although SQULView only repaints the background 
 * of the update region, it redraws the entire axes.  For better performance, 
 * only those parts of the axes that fall within the update region should 
 * be redrawn.)   
 */
{
	int		 i, j, k, l, n;
        double           p, q, r, s, t, mo;
	NXRect		 plotRect;

	if (rects == NULL) return self;

	/* paint visible area white then draw axes */
	PSsetgray(NX_WHITE);
	if (drawULBack) NXRectFill(&rects[0]);
	PSsetgray(NX_DKGRAY);

	if (drawULBnstein==YES) {
//	  for (i=0; i<TS; i++) bp[i]=0.0;
//	  for (i=0; i<NG; i++) for (j=0; j<MB; j++) {
//		if ([[theZMatrix cellAt: j: i] state]) bp[ULgtb[i]]+=1.0;
//		if ([[theOMatrix cellAt: j: i] state]) bp[ULgtb[i]+1]+=1.0;
//		}
	berncoeff();
	  ulgfram(50,3);
	  ulgdiag(50,3);
	  bulgraf(TS-1,50);
	  }

	if (drawULTrajec==YES) {
	  ulgfram(75,3);
	  ulgdiag(75,3);
	  bulgraf(TS-1,75);
	  ulstatecount(arr1,64,64);
	  for (j=0; j<KK; j++) ulllf[j]=ullf[j];
	  ulonegen(arr1,brr2,64,64);
	  if (pQULDots) ulbdot((75*ulllf[1])/4096,(75*ullf[1])/4096,3);
	    else ulbline(0.0367*((double)ullf[1]),0.0367*((double)ulllf[1]),0);
	  for (i=0; i<trajULLen; i++) {
	    for (j=0; j<KK; j++) ulllf[j]=ullf[j];
	    ulonegen(arr1,brr2,64,64);
	    if (pQULDots) ulbdot((75*ulllf[1])/4096,(75*ullf[1])/4096,3);
	  else ulbline(0.0367*((double)ullf[1]),0.0367*((double)ulllf[1]),1);
	    }
	  PSstroke();
	  }

	if (drawULRturn==YES) {
	  ulgfram(75,3);
	  ulgdiag(75,3);
	  bulgraf(TS-1,75);
	  for (i=0; i<75; i++) {
	    p=((double)i)/75.0; q=1.0-p;
	    ranblin(q,p);
	    for(j=0; j<rturnULNum; j++) ulonegen(arr1,brr2,64,64);
	    ulbdot(i,(75*ullf[1])/4096,3);
	    }
	  }

	if (drawULFreqs==YES) {
	  ulgfram(75,3);
	  for (i=0; i<75; i++) {
	    ulonegen(arr1,brr2,64,64);
	    for (j=0, k=0; j<TS; j++)
	      if (ultus[j]>57) for (l=0; l<ultus[j]; l+=58) ulbdot(i,k++,j%3);
	    ulbdot(i,(75*ullf[1])/4096,3);
	    }
	  }

	if (slowULFreqs==YES) {
	  ulgfram(75,3);
	  for (i=0; i<75; i+=3) {
	    ulonegen(arr1,brr2,64,64);
	    for (j=0, k=0; j<TS; j++)
		if (ultus[j]>57) for (l=0; l<ultus[j]; l+=58) {
		  ulbdot(i,k,j%3);
		  ulbdot(i+1,k,j%3);
		  ulbdot(i+2,k,j%3);
		  k++;
		  }
	    ulbdot(i,(75*ullf[1])/4096,3);
	    ulbdot(i+1,(75*ullf[1])/4096,3);
	    ulbdot(i+2,(75*ullf[1])/4096,3);
	    }
	  }

        if (checkULBoard==YES) {
   	  [self getBounds: &plotRect];
	  mo=(double)moULTiles;
	  j=(int)(plotRect.size.width/mo);
	  i=(int)(plotRect.size.height/mo);
	  zerline();
          for (k=0; k<i/2; k++) {
	    for (l=0; l<j/2; l++) arr1[k*j+l]=0;
            for (l=j/2; l<j; l++) arr1[k*j+l]=1;
            }
          for (k=i/2; k<i; k++) {
	    for (l=0; l<j/2; l++) arr1[k*j+l]=1;
            for (l=j/2; l<j; l++) arr1[k*j+l]=0;
            }
	  }

        if (smallULSmall==YES) {
	  mo=(double)moULTiles;
   	  [self getBounds: &plotRect];
	  j=(int)(plotRect.size.width/mo);
	  i=(int)(plotRect.size.height/mo);
	  zerline();
          for (k=i/3; k<2*(i/3); k++) for (l=j/3; l<2*(j/3); l++) 
		arr1[k*j+l]=random()%KK;
	  }

        if (drawULSmall==YES) {
	  [window setTitle: "evolution"];
   	  [self getBounds: &plotRect];
	  j=(int)(plotRect.size.width/4.0);           /* 4.0 for drawview=7 */
	  i=(int)(plotRect.size.height/4.0);
	  screen(arr1,i,j);
	  if(repULPlEv>0) ulonegen(arr1,brr2,i,j);
	  }

        if (smallULMosaic==YES) {
	  mo=(double)moULTiles;
	  [window setTitle: "mosaic"];
   	  [self getBounds:&plotRect];
	  j=(int)(plotRect.size.width/mo);
	  i=(int)(plotRect.size.height/mo);
	  for (k=0; k<moULTiles; k++) for (l=0; l<moULTiles; l++) {
	    manyULrect(k,l,moULTiles,moULTiles,i,j);
	    [window flushWindow];
	    }
	  smallULgrid(i,j);
	  }

	return self;
}

- sizeTo:(NXCoord)width :(NXCoord)height
/*
 * Ensures that whenever the SQULView is resized, the origin of its 
 * coordinate system is repositioned to the center of its area.
 */
{
//	  [self setFlipped: YES];
	[super sizeTo: width: height];
	if (drawview==4||drawview==6||drawview==7||drawview>=10) 
		{[self setDrawOrigin:0 :0]; return self;}
	else
		[self setDrawOrigin: -floor(width/2) : -floor(height/2)]; 
	return self;
}

- mouseDown:(NXEvent *) theEvent
/*
 * Responds to a message the system sends whenever the user presses the mouse
 * button when the cursor is over the SQULView.  The SQULView changes the 
 * cursor to a cross-hairs image and then starts asking for mouse-dragged or 
 * mouse-up events.  As it receives mouse-dragged events, the SQULView updates 
 * the readOut text Cell with the cursor's coordinates.  If the user releases 
 * the mouse button while the cursor is over the SQULView, the SQULView 
 * registers the point and then sends a message to its delegate notifying it 
 * of the new point.
 */
{
	int	looping = YES, oldMask;
	NXPoint	aPoint;
	NXRect	plotRect;

      /*   double  x, y,  eks, wye; */
	 double ex, wy;
	 int k;

	[crossCursor set];
	oldMask = [window addToEventMask:NX_MOUSEDRAGGEDMASK];
 				[window setDocEdited:YES];
		                aPoint = theEvent->location;
				aPoint.y = aPoint.y;
		                [self convertPoint:&aPoint fromView:nil];
                                ex=aPoint.x;
                                wy=aPoint.y;
				k=nearest(ex,wy);

//       if (theEvent->data.mouse.click==3) {} else 
//       if (theEvent->data.mouse.click==2) {} else 
	{
	[self lockFocus];
//         PSsetinstance(YES);
	do {
// 		PSnewinstance();
		aPoint = theEvent->location;
		[self convertPoint:&aPoint fromView:nil];
		aPoint.y= (plotRect.size.height-1)-aPoint.y;
                ex=aPoint.x;
                wy=aPoint.y;
				PSsetgray(NX_BLACK);
		[window flushWindow];
		if (theEvent->type == NX_MOUSEUP) {
	/* on mouse-up, register point, inform delegate, and clean up state */
			[window flushWindow];
			looping = NO;
		}	
   	} while (looping && (theEvent = 
		[NXApp getNextEvent:NX_MOUSEUPMASK|NX_MOUSEDRAGGEDMASK]));
	[self unlockFocus];

	}

	[window setEventMask:oldMask];
	[NXArrow set];
	return self;
}

@end

/*  - - - - - - - -  n x s q u l 2 1 . c  - - - - - - - - - - - - - - - - */

/* SQUL.C					*/
/* Two-Dimensional Square Lattice Life		*/
/* with binary cells, Moore neighborhood	*/
/*						*/
/* Reference:					*/
/*						*/
/* Harold V. McIntosh, 16 May 1987		*/
/* 7 July 1987 - separate rules for each state	*/
/* 8 July 1988 - binary, 3 neighborhood classes	*/
/* 9 July 1988 - hexagonal lattice 		*/
/* 20 December 1995 - fixed for lcau24 & NeXT	*/

void asctoqul() {int i0, i1, i2, i3, i4, i5, i6, i7, i8;
  for (i0=0; i0<KK; i0++)
  for (i1=0; i1<KK; i1++)
  for (i2=0; i2<KK; i2++)
  for (i3=0; i3<KK; i3++)
  for (i4=0; i4<KK; i4++)
  for (i5=0; i5<KK; i5++)
  for (i6=0; i6<KK; i6++)
  for (i7=0; i7<KK; i7++)
  for (i8=0; i8<KK; i8++)
    ulrule[i0+KK*(i1+KK*(i2+KK*(i3+KK*(i4+KK*(i5+KK*(i6+KK*(i7+KK*i8)))))))] = 
	ascrule[i0][i1][i2][i3][i4][i5][i6][i7][i8];
}

void qultoasc() {int i0, i1, i2, i3, i4, i5, i6, i7, i8;
  for (i0=0; i0<KK; i0++)
  for (i1=0; i1<KK; i1++)
  for (i2=0; i2<KK; i2++)
  for (i3=0; i3<KK; i3++)
  for (i4=0; i4<KK; i4++)
  for (i5=0; i5<KK; i5++)
  for (i6=0; i6<KK; i6++)
  for (i7=0; i7<KK; i7++)
  for (i8=0; i8<KK; i8++)
    ascrule[i0][i1][i2][i3][i4][i5][i6][i7][i8] = 
      ulrule[i0+KK*(i1+KK*(i2+KK*(i3+KK*(i4+KK*(i5+KK*(i6+KK*(i7+KK*i8)))))))];
}

/* calculate one generation of evolution */
void ulonegen(ar,br,m,n) char ar[]; int br[]; int m, n; {int i0, i1, gn;

/* count neighbors of main square */
// phrame(1); /* phase to count neighbors */

for (i0=1; i0<m-1; i0++)
for (i1=1; i1<n-1; i1++) {
  gn=ar[(i0-1)*n+i1+1];
  gn=KK*gn+ar[i0*n+i1+1];
  gn=KK*gn+ar[(i0+1)*n+i1+1];
  gn=KK*gn+ar[(i0+1)*n+i1];
  gn=KK*gn+ar[(i0+1)*n+i1-1];
  gn=KK*gn+ar[i0*n+i1-1];
  gn=KK*gn+ar[(i0-1)*n+i1-1];
  gn=KK*gn+ar[(i0-1)*n+i1];
  gn+=GH*ar[i0*n+i1];
  br[i0*n+i1]=gn;
  }

/* count neighbors for left edge (i1=0) */

for (i0=1; i0<m-1; i0++) {
  gn=ar[(i0-1)*n+1];
  gn=KK*gn+ar[i0*n+1];
  gn=KK*gn+ar[(i0+1)*n+1];
  gn=KK*gn+ar[(i0+1)*n];
  gn=KK*gn+ar[(i0+1)*n+n-1];
  gn=KK*gn+ar[i0*n+n-1];
  gn=KK*gn+ar[(i0-1)*n+n-1];
  gn=KK*gn+ar[(i0-1)*n];
  gn+=GH*ar[i0*n];
  br[i0*n]=gn;
  }

/* count neighbors for right edge (i1=n-1) */

for (i0=1; i0<m-1; i0++) {
  gn=ar[(i0-1)*n];
  gn=KK*gn+ar[i0*n];
  gn=KK*gn+ar[(i0+1)*n];
  gn=KK*gn+ar[(i0+1)*n+(n-1)];
  gn=KK*gn+ar[(i0+1)*n+(n-1)-1];
  gn=KK*gn+ar[i0*n+(n-1)-1];
  gn=KK*gn+ar[(i0-1)*n+(n-1)-1];
  gn=KK*gn+ar[(i0-1)*n+(n-1)];
  gn+=GH*ar[i0*n+(n-1)];
  br[i0*n+(n-1)]=gn;
  }

/* count neighbors for top edge (i0=0) */

for (i1=1; i1<n-1; i1++) {
  gn=ar[(m-1)*n+i1+1];
  gn=KK*gn+ar[i1+1];
  gn=KK*gn+ar[n+i1+1];
  gn=KK*gn+ar[n+i1];
  gn=KK*gn+ar[n+i1-1];
  gn=KK*gn+ar[i1-1];
  gn=KK*gn+ar[(m-1)*n+i1-1];
  gn=KK*gn+ar[(m-1)*n+i1];
  gn+=GH*ar[i1];
  br[i1]=gn;
  }

/* count neighbors of bottom edge (i0=m-1) */

for (i1=1; i1<n-1; i1++) {
  gn=ar[(m-2)*n+i1+1];
  gn=KK*gn+ar[(m-1)*n+i1+1];
  gn=KK*gn+ar[i1+1];
  gn=KK*gn+ar[i1];
  gn=KK*gn+ar[i1-1];
  gn=KK*gn+ar[(m-1)*n+i1-1];
  gn=KK*gn+ar[(m-2)*n+i1-1];
  gn=KK*gn+ar[(m-2)*n+i1];
  gn+=GH*ar[(m-1)*n+i1];
  br[(m-1)*n+i1]=gn;
  }

/* count neighbors for upper left hand corner */
/* i0 = 0, i1 = 0			      */

  gn=ar[(m-1)*n+1];
  gn=KK*gn+ar[1];
  gn=KK*gn+ar[n+1];
  gn=KK*gn+ar[n];
  gn=KK*gn+ar[n+n-1];
  gn=KK*gn+ar[n-1];
  gn=KK*gn+ar[m*n-1];
  gn=KK*gn+ar[(m-1)*n];
  if (ar[0]==1) gn+=GH;
  br[0]=gn;

/* count neighbors for lower left hand corner */
/* i0 = m-1, i1 = 0			      */

  gn=ar[(m-2)*n+1];
  gn=KK*gn+ar[(m-1)*n+1];
  gn=KK*gn+ar[1];
  gn=KK*gn+ar[0];
  gn=KK*gn+ar[n-1];
  gn=KK*gn+ar[(m-1)*n+n-1];
  gn=KK*gn+ar[(m-2)*n+n-1];
  gn=KK*gn+ar[(m-2)*n];
  if (ar[(m-1)*n]==1) gn+=GH;
  br[(m-1)*n]=gn;

/* count neighbors for upper right hand corner */
/* i0 = 0, i1 = n-1			       */

  gn=ar[(m-1)*n];
  gn=KK*gn+ar[0];
  gn=KK*gn+ar[n];
  gn=KK*gn+ar[n+n-1];
  gn=KK*gn+ar[n+n-2];
  gn=KK*gn+ar[n-2];
  gn=KK*gn+ar[m*n-2];
  gn=KK*gn+ar[m*n-1];
  if (ar[n-1]==1) gn+=GH;
  br[n-1]=gn;

/* count neighbors for lower right hand corner */
/* i0 = m-1, i1 = n-1			       */

  gn=ar[(m-2)*n];
  gn=KK*gn+ar[(m-1)*n];
  gn=KK*gn+ar[0];
  gn=KK*gn+ar[n-1];
  gn=KK*gn+ar[n-2];
  gn=KK*gn+ar[(m-1)*n+n-2];
  gn=KK*gn+ar[(m-2)*n+n-2];
  gn=KK*gn+ar[(m-2)*n+n-1];
  if (ar[(m-1)*n+n-1]==1) gn+=GH;
  br[(m-1)*n+n-1]=gn;

// phrame(2); /* phase to update cells */
for (i0=0; i0<KK; i0++)   ullf[i0] =0;
for (i0=0; i0<TS; i0++)   ultus[i0]=0;
for (i0=0; i0<DS; i0++)   ulrus[i0]=0;
for (i0=0; i0<2*NG; i0++) ulgus[i0]=0;
for (i0=0; i0<m; i0++) for (i1=0; i1<n; i1++) {
  ar[i0*n+i1]=ulrule[br[i0*n+i1]]-'0';
  ullf[ar[i0*n+i1]]+=1;
  ulgus[gotb[br[i0*n+i1]]]+=1;
  ultus[tc[br[i0*n+i1]]]+=1;
  ulrus[br[i0*n+i1]]++;
  }
// phrame(3); /* phase to show results  */
// printf("live %5d/ %5d total\n",lf,m*n);
}	  /* end subroutine ulonegen */
    
/*find fraction of live cells in arry[m][n] */
void ulstatecount(aa,m,n) char *aa; int m, n; {int i0, i1;
  for (i0=0; i0<KK; i0++) ullf[i0]=0;
  for (i0=0; i0<m; i0++) for (i1=0; i1<n; i1++) ullf[aa[n*i0+i1]]+=1;
}

/* display a screen of evolution */
void screen(aa,m,n) char *aa; int m, n; {int i, j;
  for (j=0; j<m; j++) for (i=0; i<n; i++) videodot(j,i,aa[n*j+i]);}

// /* display a phrame around the cell array */
// phrame(l) int l; {int j;
// for (j=0; j<QL; j++) {
//   uladot(0,j,l);
//   uladot(QL-1,j,l);
//   uladot(j,0,l);
//   uladot(j,QL-1,l);
//   }
// }

/* make a phrame for the fixpoint graph  */
/* n     = dimension of phrame           */
void ulgfram(n) int n; {int i;
for (i=0; i<=n; i++) {
  ulbdot(i,0,0);
  ulbdot(i,n,0);
  ulbdot(n,i,0);
  ulbdot(0,i,0);
  }
for (i=0; i<=2*n; i+=15) {
  ulbdot(i/2,0,3);
  ulbdot(i/2,n,3);
  ulbdot(n,i/2,3);
  ulbdot(0,i/2,3);
  }
}

/* put a diagonal in the fixpoint graph */
void ulgdiag(n) int n; {int i; for (i=0; 2*i<n; i++) ulbdot(2*i,2*i,3);}

/* graph Bernstein polynomial */
void bulgraf(k,n) int k, n; {int i; double ulburn(), en, p;
en=(double)(n);
for (i=0; i<n; i++) {
  p=((double)(i))/en;
  ulbdot(i,(int)(en*ulburn(p,k)),1);
  }
}

/* evaluate the nth degree Bernstein polynomial at point p */
double ulburn(p,n) double p; int n; {double q, s, x, r; int i;
if (p>0.99) return bp[n];
q=1.0-p; r=p/q; s=0.0; x=1.0;
for (i=0; i<n;  i++) x*=q;
for (i=0; i<=n; i++, x*=r) s+=bp[i]*x;
s*=0.9998;
return(s+0.0001);
}

/* ulbdot(i,j,l) - place dot on Bernstein screen at (i,j) in color l */
void ulbdot(i,j,l) int i, j, l; {videodot(j,i,l);}

/* ulbline(x,y,l) - place line on Bernstein screen to (x,y); l=first */
void ulbline(x,y,l) double x, y; int l; 
	{if (l==0) PSmoveto(y,150.0-x); else PSlineto(y,150.0-x);}

/* display Wolfram rule number of planar cross section */
/* calculate LCAU21 wru from a SQUL21 argument rule    */
void wqusscrrul(c,ru) char *c, ru[KK][KK][KK][KK][KK][KK][KK][KK][KK]; {
char ur[KK][KK][KK]; int i0, i1, i2;
  for (i0=0; i0<KK; i0++) for (i1=0; i1<KK; i1++) for (i2=0; i2<KK; i2++)
    ur[i0][i1][i2]=ru[i0][i0][i0][i1][i2][i2][i2][i1][i1];
  wrru21(c,ur);
  }

/* calculate wru from an argument rule	*/
void wrru21(c,ru) char *c, ru[KK][KK][KK]; {int  k, i0, i1, i2;
  k=0;
  for (i0=KK-1; i0>=0; i0--)
  for (i1=KK-1; i1>=0; i1--)
  for (i2=KK-1; i2>=0; i2--)
    k=2*k+(ru[i0][i1][i2]-'0');
  sprintf(c,"%3d",k);
}

/* generate the Golay code table */
golta() {
int i, j, k;
int i0, i1, i2, i3, i4, i5, i6, i7;
k=0;
for (i7=0; i7<KK; i7++)
for (i6=0; i6<KK; i6++)
for (i5=0; i5<KK; i5++)
for (i4=0; i4<KK; i4++)
for (i3=0; i3<KK; i3++)
for (i2=0; i2<KK; i2++)
for (i1=0; i1<KK; i1++)
for (i0=0; i0<KK; i0++) {
i=binq(i0,i1,i2,i3,i4,i5,i6,i7);
j=binq(i2,i3,i4,i5,i6,i7,i0,i1);
if (j<i) i=j;
j=binq(i4,i5,i6,i7,i0,i1,i2,i3);
if (j<i) i=j;
j=binq(i6,i7,i0,i1,i2,i3,i4,i5);
if (j<i) i=j;
j=binq(i0,i7,i6,i5,i4,i3,i2,i1);
if (j<i) i=j;
j=binq(i6,i5,i4,i3,i2,i1,i0,i7);
if (j<i) i=j;
j=binq(i4,i3,i2,i1,i0,i7,i6,i5);
if (j<i) i=j;
j=binq(i2,i1,i0,i7,i6,i5,i4,i3);
if (j<i) i=j;
j=binq(i0,i1,i2,i3,i4,i5,i6,i7);
if (i==j) {
  gc[i]=k;
  sc[k][i0]++; sc[k][i1]++; sc[k][i2]++; sc[k][i3]++;
  wc[k]=1;
  nr[k]=j;
  k++;
} else {
  gc[j]=gc[i]; wc[gc[i]]++;
  }
}
}

/* calculate binary equivalent */
int binq(i0,i1,i2,i3,i4,i5,i6,i7)
int i0, i1, i2, i3, i4, i5, i6, i7; {
return i0+KK*(i1+KK*(i2+KK*(i3+KK*(i4+KK*(i5+KK*(i6+KK*i7))))));}

/* generate the transitions for Conway's Life */
void conlif() {int i;
  for (i=0; i<DS; i++) ulrule[i]='0';
  ulrule[5]='1';  ulrule[8]='1';  ulrule[9]='1';  ulrule[10]='1';
  ulrule[13]='1'; ulrule[14]='1'; ulrule[19]='1'; ulrule[20]='1';
  ulrule[22]='1'; ulrule[23]='1'; ulrule[56]='1'; ulrule[59]='1';
  ulrule[60]='1'; ulrule[61]='1'; ulrule[64]='1'; ulrule[65]='1';
  ulrule[70]='1'; ulrule[71]='1'; ulrule[73]='1'; ulrule[74]='1';
  ulrule[54]='1'; ulrule[55]='1'; ulrule[57]='1'; ulrule[58]='1';
  ulrule[63]='1'; ulrule[69]='1';
printf("conlif\n");
}

/* generate the transitions for a generalization of Rule 110 */
void tiling() {int i;
  for (i=0; i<DS; i++) ulrule[i]='1';
  ulrule[0]='0'; ulrule[79]='0'; ulrule[335]='0'; ulrule[511]='0';
  qultoasc();
printf("tiling\n");
}

/* end SQULView.m */
