
/* 
 * VN4DView.m -- Implementation file for the VN4DView class 
 *
 * You may freely copy, distribute, and reuse the code in this example.
 * NeXT disclaims any warranty of any kind, expressed or implied, as to 
 * its fitness for any particular use, nor even that it has a use.
 *
 */

#import "VN4DView.h"
#import "drawingFuncs.h"
#import "nxlcau24.h"
#import <math.h> 
#import <stdlib.h>
#import <string.h>

#define MB 16				/* maximum binomial coefficient */
#define NG 16				/* number of Golay Classes	*/
#define GH    (DS/KK)		/* DS/KK= half number of neighborhoods	*/

extern char ascrule[KK][KK][KK][KK][KK][KK][KK][KK][KK];
extern char auxrule[KK][KK][KK][KK][KK][KK][KK][KK][KK];
extern char ttrule[];			/* evolutionary rule */
char   golRule[KK*NG];	/* evolutionary rule       */
char   sqrule[DS];	/* evolutionary rule       */
char   qarule[DS];	/* auxiliary rule flags    */

extern char    arr1[], arr2[];
       int             brr2[AL];
extern double  bp[];			/* Bernstein coefficients    */
extern int     tc[];			/* totalistic key array      */
int    llf[KK];				/* save number of live cells */
int    lf[KK];				/* number of live cells	   */
int    gus[2*NG];			/* golay use statistics    */
int    rus[DS];				/* rule use statistics     */
int    tus[TS];				/* totalistic use stats    */

int    totb[DS];
int    gotb[DS];

extern NXColor colst[KK];    		/* Array for state colors    */

int    cellNood=NO;
int    checkBoard=NO;
int    drawARBack=YES;
int    drawBnstein=NO;
int    drawVNMosaic=NO;
int    drawFreqs=NO;
int    drawVNPlane=NO;
int    drawQuency=NO;
int    drawRturn=NO;
int    drawSmall=NO;
int    drawTrajec=NO;
int    slowFreqs=NO;
int    smallSmall=NO;
int    smallMosaic=NO;

int    lastPlane=0;			/* 0 = none, 1 = evol, 2 = mosaic */
int    lastTraj=0;			/* 0 = none, 1 = traj, 2 = return */
int    moTiles=3;
int    pDots=NO;
int    trajLen=10;
int    repPlEv=0;
int    rturnNum=1;
int    skipThree=NO;
int    stretch=125;

int    totpmut[TS] = {0,1,2,3,4,5,6,7,8,9};
int    gtb[NG] = {0,1,2,2,3,3,4,4,5,6};
double ttb[TS] = {1.0,9.0,36.0,84.0,126.0,126.0,84.0,36.0,9.0,1.0};
NXColor totcolst[TS];    		/* Array for totalistic colors   */

extern int    drawview;
extern int    co, dn;

void    arbdot(int i, int j, int l);
void    arbline(double x, double y, int l);
void    argfram(int n, int l);
void    argdiag(int n, int l);
void    arscreen(char *arr1, int k, int l, int m, int n, int q, int r);
void    asctobin();
void	asctoqar();
void	asczero();
void    bargraf(int m, int n);
void    aronegen(char *aa, int *bb, int k, int l, int m, int n); 
int     istot();
int     ix2(int i, int j);
int     ix3(int i, int j, int k);
int     inx4(int i0, int i1, int i2, int i3, int j, int k, int l, int m);
void    manyrect(int i, int j, int k, int l, int m, int n, int q, int r);
int     nearest(double x, double y);
void    onegen(int n);
void    ranblin(double p1, double p2);
void    renewarry();
void    savearry();
void    screen(char *aa, int m, int n);
void    smallgrid(int m, int n);
void    statecount(char *aa, int k, int l, int m, int n);
void    suggrule(int i);
int     totonu(int n);
void    videodot(int row, int col, int color);
void    wrru(char *c, char ru[KK][KK][KK][KK][KK][KK][KK][KK][KK]);
void    warsscrrul(char *c, char ru[KK][KK][KK][KK][KK][KK][KK][KK][KK]);
void    zerline();

@implementation VN4DView
 
/* -------------------------------- S Q A R -------------------------------- */

/* put up the principal window for the SqarView. Since sqar.nib is a 	  */
/* secondary .nib file, it is not loaded until there is a demand, which   */
/* would usually arise from the "Sqar" option in the main menu.	   	  */

- loadSqar:sender {
  if (sqarWindow==nil) {
    if (![NXApp loadNibSection: "vn4d.nib" owner: self withNames: NO]) 
	{return nil;}
    }
  dn=4;
  [SqarWindow makeKeyAndOrderFront: sender];
  [sqarWindow orderFront: nil];
  [GolayPanel orderFront: nil];
  [self copyRule: sender];
return self;
}

void drawVNsquare() {int i, j;
  for (i=0; i<128; i++) for (j=0; j<128; j++) videodot(j,i,arr1[i*128+j]+1);
}

void drawVNcube() {int i, j, k;
  for (i=0; i<32; i++) for (j=0; j<32; j++) for (k=0; k<16; k++) 
    videodot(j+32*(k/4),i+32*(k%4),arr1[ix3(i,j,k)]+1);
}

void manyVNsquare(i,j) int i, j; {int k, l;
  for (k=0; k<128; k++) for (l=0; l<128; l++)  
    videodot(k+128*i,l+128*j,arr1[ix2(k,l)]+1);
}

void drawVNgrid() {int i;
	  PSsetgray(NX_BLACK); 
	  PSsetlinewidth(0.1);
  for (i=0; i<5; i++) {
  PSmoveto((double)(128.0*i),0.0); PSlineto((double)(128.0*i),512.0);}
  for (i=0; i<5; i++) {
  PSmoveto(0.0,(double)(128.0*i)); PSlineto(512.0,(double)(128.0*i));}
  PSstroke();
}

- printSQAR:sender {
   if (keyWindow==SqarWindow) 
	{[SqarWindow smartPrintPSCode: self];}
   else if (keyWindow==GolayPanel) 
	{[GolayPanel smartPrintPSCode: self];}
   else if (keyWindow==planeWindow) {
    switch (lastPlane) {
      case 0: break;
      case 1: drawVNPlane=YES;   break;
      case 2: drawVNMosaic=YES;  break;
      case 3: drawSmall=YES;   break;
      case 4: smallMosaic=YES; break;
      default: break;}
    [planeWindow smartPrintPSCode: self];
    switch (lastPlane) {
      case 0: break;
      case 1: drawVNPlane=NO;   break;
      case 2: drawVNMosaic=NO;  break;
      case 3: drawSmall=NO;   break;
      case 4: smallMosaic=NO; break;
      default: break;}
    }
return self;
}

- windowDidBecomeKey:sender {
	keyWindow=sender;
	[menuCopy setTarget: self];
	[menuCopy setAction: @selector(copySQAR:)];
	[menuCopy setEnabled: YES];
	[menuPrint setTarget: self];
	[menuPrint setAction: @selector(printSQAR:)];
	[menuPrint setEnabled: YES];
return self;
}

- windowDidResignKey:sender {
	[menuCopy setEnabled: NO];
	[menuPrint setEnabled: NO];
return self;
}

- copySqarView:sender {
NXRect zbounds;
id     pBoard;
  pBoard = [Pasteboard new];
  [pBoard declareTypes: &NXPostScriptPboardType num: 1 owner: self];
  [SqarView getBounds: &zbounds];
  [SqarView writePSCodeInside: &zbounds to: pBoard]; 
return self;
}

- copyGolayPanel:sender {
NXRect zbounds;
id     pBoard;
  pBoard = [Pasteboard new];
  [pBoard declareTypes: &NXPostScriptPboardType num: 1 owner: self];
  [GolayView getBounds: &zbounds];
  [GolayView writePSCodeInside: &zbounds to: pBoard]; 
return self;
}

- copyQuency:sender {
NXRect zbounds;
id     pBoard;
  drawQuency = YES;
  drawview = 14;
  drawARBack = NO;
  pBoard = [Pasteboard new];
  [pBoard declareTypes: &NXPostScriptPboardType num: 1 owner: self];
  if ([justTheView state]) {
    [freqView getBounds: &zbounds];
    [freqView writePSCodeInside: &zbounds to: pBoard]; 
    } else {
    [[freqWindow contentView] getBounds: &zbounds];
    [[freqWindow contentView] writePSCodeInside: &zbounds to: pBoard]; 
    }
  drawARBack = YES;
  drawQuency = NO;
return self;
}

- copyStretch:sender {
NXRect zbounds;
id     pBoard;
  drawview = 14;
  stretch=375;
  drawQuency = YES;
  drawARBack = NO;
  if ([reStock state]) savearry();
  pBoard = [Pasteboard new];
  [pBoard declareTypes: &NXPostScriptPboardType num: 1 owner: self];
    [stretchView getBounds: &zbounds];
    [stretchView writePSCodeInside: &zbounds to: pBoard]; 
  if ([reStock state]) renewarry();
  drawARBack = YES;
  drawQuency = NO;
  stretch=125;
return self;
}

- copyWhoosit: (int *) flag {
NXRect zbounds;
id     pBoard;
  *flag = YES;
  drawview=13;
  drawARBack=NO;
  pBoard = [Pasteboard new];
  [pBoard declareTypes: &NXPostScriptPboardType num: 1 owner: self];
  [trajView getBounds: &zbounds];
  [trajView writePSCodeInside: &zbounds to: pBoard]; 
  drawARBack=YES;
  *flag = NO;
return self;
}

- copyWhatsit: (int *) flag {
NXRect zbounds;
id     pBoard;
  *flag = YES;
  drawARBack=NO;
  pBoard = [Pasteboard new];
  [pBoard declareTypes: &NXPostScriptPboardType num: 1 owner: self];
  [planeView getBounds: &zbounds];
  [planeView writePSCodeInside: &zbounds to: pBoard]; 
  drawARBack=YES;
  *flag = NO;
return self;
}

- copySQAR:sender {
   if (keyWindow==SqarWindow) {[self copySqarView: self];}
   else if (keyWindow==GolayPanel) {[self copyGolayPanel: self];}
   else if (keyWindow==trajWindow) {
     savearry();
     switch (lastTraj) {
          case 0: break;
          case 1: [self copyWhoosit: &drawTrajec]; break;
          case 2: [self copyWhoosit: &drawRturn];  break;
          case 3: [self copyWhoosit: &drawFreqs];  break;
          case 4: [self copyWhoosit: &slowFreqs];  break;
          default: break;}
     renewarry();}
   else if (keyWindow==planeWindow) switch (lastPlane) {
     case 0: break;
     case 1: savearry(); [self copyWhatsit: &drawVNPlane]; renewarry();   break;
     case 2: savearry(); [self copyWhatsit: &drawVNMosaic]; renewarry();  break;
     case 3: drawview=7; repPlEv=1; [self copyWhatsit: &drawSmall]; break;
     case 4: savearry(); drawview=11; [self copyWhatsit: &smallMosaic]; 
		renewarry(); break;
     default: break;}
   else if (keyWindow==freqWindow)    [self copyQuency: self];
   else if (keyWindow==stretchWindow) [self copyStretch: self];
return self;
}

- drawPlEvol:sender {int i;
  repPlEv=[[sender selectedCell] tag];
  drawview=7;
  drawVNPlane=YES;
  asctobin();
  if (repPlEv==0) [self display]; else for (i=0; i<repPlEv; i++) {
     [genEv setIntValue: i];
     onegen(AL);
     [self display];
     }
  [genEv setIntValue: repPlEv];
  drawVNPlane=NO;
  lastPlane=1;
return self;
}

- printPlEvol:sender {
    repPlEv=0;
    drawview=7;
    drawVNPlane=YES;
    [keyWindow smartPrintPSCode: self];
    drawVNPlane=NO;
return self;
}

- drawMosaic:sender {
    drawview=11;
    drawVNMosaic=YES;
    [self display];
    drawVNMosaic=NO;
    lastPlane=2;
return self;
}

- printMosaic:sender {
    drawview=11;
    drawVNMosaic=YES;
    [keyWindow smartPrintPSCode: self];
    drawVNMosaic=NO;
return self;
}

- showProbability:sender {int i; double pp, p[KK];
  for (i=0; i<KK; i++) p[i]=[[sender cellAt: i: 0] doubleValue];
  for (pp=0, i=0; i<KK; i++) pp+=p[i];
  for (i=0; i<KK; i++) {
    [[pSliders cellAt: i: 0] setDoubleValue: p[i]/pp];
    [[pWindows cellAt: i: 0] setDoubleValue: p[i]/pp];
    }
return self;
}

- randomLine:sender {double p[KK]; int i;
  for (i=0; i<KK; i++) p[i]=[[pSliders cellAt:i:0] doubleValue];
  ranblin(p[0],p[1]);
return self;
}

- sCellLine:sender {
  smallSmall=YES;
  [planeView display];
  smallSmall=NO;
return self;
}

- uniformLine:sender {ranblin(0.5,0.5); return self;}

- checkerBoard:sender {
  checkBoard=YES;
  [planeView display];
  checkBoard=NO;
return self;
}

- uprule:sender {suggrule(0); [self anyrule: self]; return self;}
- lerule:sender {suggrule(1); [self anyrule: self]; return self;}
- dorule:sender {suggrule(3); [self anyrule: self]; return self;}
- rirule:sender {suggrule(2); [self anyrule: self]; return self;}
- idrule:sender {suggrule(4); [self anyrule: self]; return self;}
- mjrule:sender {suggrule(5); [self anyrule: self]; return self;}

- anyrule:sender {
   [self copyRule: self];
   [rulePanel setAllRules: self];
return self;}

void suggrule(i) int i; {int j, i0, i1, i2, i3, i4, i5, i6, i7, i8;
  for (i0=0; i0<KK; i0++)
  for (i1=0; i1<KK; i1++)
  for (i2=0; i2<KK; i2++)
  for (i3=0; i3<KK; i3++)
  for (i4=0; i4<KK; i4++)
  for (i5=0; i5<KK; i5++)
  for (i6=0; i6<KK; i6++)
  for (i7=0; i7<KK; i7++)
  for (i8=0; i8<KK; i8++) {
    switch (i) {
      case 0: j=i2; break;
      case 1: j=i3; break;
      case 2: j=i1; break;
      case 3: j=i0; break;
      case 4: j=i4; break;
      case 5: j=0; if (i0+i1+i2+i3+i4+i5+i6+i7+i8>4) j=1; break;
      default: j=0; break;}
    ascrule[i0][i1][i2][i3][i4][i5][i6][i7][i8] = '0'+j;
    }
}

/* --------------------------- E V O L U T I O N --------------------------- */

- drawSmEvol:sender {int k;
    drawview=7;
    drawSmall=YES;
    asctoqar();
    if ([reStore state]) savearry();
    sscanf([[sender selectedCell] title],"%d",&repPlEv);
    [self display];
    if (repPlEv>0) for (k=0; k<repPlEv; k++) [self display];
    if ([reStore state]) renewarry();
    drawSmall=NO;
    lastPlane=3;
return self;
}

- printSmEvol:sender {
    drawview=7;
    drawSmall=YES;
    repPlEv=0;
    [keyWindow smartPrintPSCode: self];
    drawSmall=NO;
return self;
}

- drawSmMosaic:sender {
    cellNood=[cellNhood state];
    moTiles=[moTies selectedRow]+2;
    drawview=11;
    smallMosaic=YES;
    asctoqar();
    savearry();
    [self display];
    renewarry();
    smallMosaic=NO;
    lastPlane=4;
return self;
}

- printSmMosaic:sender {
    drawview=11;
    smallMosaic=YES;
    savearry();
    [keyWindow smartPrintPSCode: self];
    renewarry();
    smallMosaic=NO;
return self;
}

- drawTraj:sender {
    sscanf([[sender selectedCell] title],"%d",&trajLen);
    pDots=[probDots state];
    drawview=13;
    drawTrajec=YES;
    asctoqar();
    savearry();
    [trajView display];
    renewarry();
    drawTrajec=NO;
    lastTraj=1;
return self;
}

- printTraj:sender {
    drawview=13;
    drawTrajec=YES;
    savearry();
    [keyWindow smartPrintPSCode: self];
    renewarry();
    drawTrajec=NO;
return self;
}

- drawRtrn:sender {
    sscanf([[sender selectedCell] title],"%d",&rturnNum);
    drawview=13;
    drawRturn=YES;
    asctoqar();
    savearry();
    [trajView display];
    renewarry();
    drawRturn=NO;
    lastTraj=2;
return self;
}

- printRtrn:sender {
    drawview=13;
    drawRturn=YES;
    savearry();
    [keyWindow smartPrintPSCode: self];
    renewarry();
    drawRturn=NO;
return self;
}

- drawFreq:sender {
//    sscanf([[sender selectedCell] title],"%d",&rturnNum);
    drawview=13;
    drawFreqs=YES;
    asctoqar();
    savearry();
    [trajView display];
    renewarry();
    drawFreqs=NO;
    lastTraj=3;
return self;
}

- printFreq:sender {
    drawview=13;
    drawFreqs=YES;
    savearry();
    [keyWindow smartPrintPSCode: self];
    renewarry();
    drawFreqs=NO;
return self;
}

- slowFreq:sender {
//    sscanf([[sender selectedCell] title],"%d",&rturnNum);
    drawview=13;
    slowFreqs=YES;
    asctoqar();
    savearry();
    [trajView display];
    renewarry();
    slowFreqs=NO;
    lastTraj=4;
return self;
}

- printSlowFreq:sender {
    drawview=13;
    slowFreqs=YES;
    savearry();
    [keyWindow smartPrintPSCode: self];
    renewarry();
    slowFreqs=NO;
return self;
}

- drawFrequency:sender {
printf("drawing Frequency\n");
    skipThree=[eachAndAll state];
    totpmut[0]=[freqPerm0 selectedCol];
    totpmut[1]=[freqPerm1 selectedCol];
    totpmut[2]=[freqPerm2 selectedCol];
    totpmut[3]=[freqPerm3 selectedCol];
    totpmut[4]=[freqPerm4 selectedCol];
    totpmut[5]=[freqPerm5 selectedCol];
    totpmut[6]=[freqPerm6 selectedCol];
    totpmut[7]=[freqPerm7 selectedCol];
    totpmut[8]=[freqPerm8 selectedCol];
    totpmut[9]=[freqPerm9 selectedCol];
    totcolst[0]=[freqColor0 color];
    totcolst[1]=[freqColor1 color];
    totcolst[2]=[freqColor2 color];
    totcolst[3]=[freqColor3 color];
    totcolst[4]=[freqColor4 color];
    totcolst[5]=[freqColor5 color];
    totcolst[6]=[freqColor6 color];
    totcolst[7]=[freqColor7 color];
    totcolst[8]=[freqColor8 color];
    totcolst[9]=[freqColor9 color];
    drawview=14;
    asctoqar();
    if ([reStock state]) savearry();
    if ([stretchOut state]) {
	[stretchWindow makeKeyAndOrderFront: sender];
        drawQuency=YES;
	stretch=375;
	[stretchView display];
	stretch=125;
	} else {drawQuency=YES; [freqView display];}
    if ([reStock state]) renewarry();
    drawQuency=NO;
return self;
}

- printFrequency:sender {
    drawview=14;
    drawQuency=YES;
    savearry();
    [keyWindow smartPrintPSCode: self];
    renewarry();
    drawQuency=NO;
return self;
}

/* draw the rectangle with coordinates (i,j) in a   */
/* mosaic whose individual rectangles measure (k,l) */
/* where the overall array has dimension (k,l,m,n)  */
/* and the plane with coordinates (q,r) is chosen   */
void manyrect(i,j,k,l,m,n,q,r) int i, j, k, l, m, n, q, r; {int ii, jj, pj;
  asctoqar();
  for (ii=0; ii<k; ii++) for (jj=0; jj<l; jj++) 
    if (cellNood) {
		 pj=totpmut[tc[brr2[ii+l*jj]]];
		 NXSetColor(totcolst[pj]);
	         videodot(i*k+ii,j*l+jj,5);
		 }
    else videodot(i*k+ii,j*l+jj,arr1[inx4(ii,jj,q,r,k,l,m,n)]);
  aronegen(arr1,brr2,k,l,m,n);
}

/* m = high/moTiles, n = wide/moTiles */
void smallgrid(m,n) int m, n; {int i; double em, en, mo;
  em=(double)m;
  en=(double)n;
  mo=(double)moTiles;
	  PSsetgray(NX_BLACK); 
	  PSsetlinewidth(0.1);
  for (i=0; i<moTiles+1; i++) {
  PSmoveto((double)(en*i),0.0); PSlineto((double)(en*i),mo*em);}
  for (i=0; i<moTiles+1; i++) {
  PSmoveto(0.0,(double)(em*i)); PSlineto(mo*en,(double)(em*i));}
  PSstroke();
}

/* -----------------------  D E F I N E   R U L E S  ----------------------- */

/* - - - - - - - -   B E R N S T E I N   H I S T O G R A M S   - - - - - - */

- drawBern:sender {
    drawview=12;
    drawBnstein=YES;
    [bernView display];
    drawBnstein=NO;
    [window setDocEdited:YES];
return self;
}

- selfBern:sender {
    drawview=12;
    drawBnstein=YES;
    [self display];
    drawBnstein=NO;
    [window setDocEdited:YES];
return self;
}

- printRule:sender {
   drawview=12;
   drawBnstein=YES;
   [keyWindow printPSCode:self];
   drawBnstein=NO;
return self;
}

- enterRule:sender {id cell; char *c, ch[WNML]; int i, j;
char    tiempo[30], fecha[30];
struct  tm  *timeptr;
time_t  secsnow;

   time(&secsnow);
   timeptr = localtime(&secsnow);
   sprintf(tiempo,"%02d:%02d:%02d\n",
	timeptr->tm_hour, 
	timeptr->tm_min,
        timeptr->tm_sec);
   [tiTime setStringValue: tiempo];
   sprintf(fecha,"%d-%d-19%02d\n",
	((timeptr->tm_mon) + 1), 
	timeptr->tm_mday,
        timeptr->tm_year);
   [tiDat setStringValue: fecha];

    for (i=0; i<NG; i++) for (j=0; j<MB; j++) {
      cell=[theZMatrix cellAt: j: i];
      if ([cell isEnabled]) {
	c=[cell title];
        ascrule[c[0]-'0']
		[c[1]-'0']
		[c[2]-'0']
		[c[3]-'0']
		[c[4]-'0']
		[c[5]-'0']
		[c[6]-'0']
		[c[7]-'0']
		[0]         =[cell intValue]+'0';
	}
      cell=[theOMatrix cellAt: j: i];
      if ([cell isEnabled]) {
	c=[cell title];
        ascrule[c[0]-'0']
		[c[1]-'0']
		[c[2]-'0']
		[c[3]-'0']
		[c[4]-'0']
		[c[5]-'0']
		[c[6]-'0']
		[c[7]-'0']
		[1]         =[cell intValue]+'0';
	}
      }
      wrru(ch,ascrule);
      [ruleNumber setStringValue: ch];
      if (istot()) {
		[tRuleNumber setIntValue: totonu(0)];
		for (i=0; i<TS; i++) 
		  [[theToMatrix cellAt: 0: i] setIntValue: ttrule[i]-'0'];
		}
      else [tRuleNumber setStringValue: "enter totalistic #"];
      [SqarWindow setDocEdited: NO];
      [rulePanel setAllRules: self];
      [bernView selfBern: nil];
return self;
}

- resetRule:sender {int i, j; 
    for (i=0; i<NG; i++) for (j=0; j<MB; j++) {
	[[theOMatrix cellAt: j: i] setIntValue: NO];
	[[theZMatrix cellAt: j: i] setIntValue: NO];
	}
return self;
}

- copyRule:sender {
id cell;
char *c, ch[WNML];
int i0, i1, i2, i3, j0, j1, j2, j3, ii, jj;
int i;
    for (i0=0; i0<KK; i0++) for (i1=0; i1<KK; i1++) 
    for (i2=0; i2<KK; i2++) for (i3=0; i3<KK; i3++) 
    for (j0=0; j0<KK; j0++) for (j1=0; j1<KK; j1++) 
    for (j2=0; j2<KK; j2++) for (j3=0; j3<KK; j3++) {
      ii=i0+KK*(i1+KK*(i2+KK*i3));
      jj=j0+KK*(j1+KK*(j2+KK*j3));
      cell=[theZMatrix cellAt: jj: ii];
      [cell setIntValue: ascrule[i0][i1][i2][i3][j0][j1][j2][j3][0]-'0'];
      cell=[theOMatrix cellAt: jj: ii];
      [cell setIntValue: ascrule[i0][i1][i2][i3][j0][j1][j2][j3][1]-'0'];
      }
      wrru(ch,ascrule);
      [ruleNumber setStringValue: ch];
      warsscrrul(ch,ascrule);
      [xsRuleNumber setStringValue: ch];
      if (istot()) {
		[tRuleNumber setIntValue: totonu(0)];
		for (i=0; i<TS; i++) 
		  [[theToMatrix cellAt: 0: i] setIntValue: ttrule[i]-'0'];
		}
      else [tRuleNumber setStringValue: "enter totalistic #"];
      [SqarWindow setDocEdited: NO];
      [bernView selfBern: nil];
return self;
}

- readRule:sender 
    {[rulePanel readWRule: sender]; [self copyRule: self]; return self;}

- readTRule:sender 
    {[rulePanel readTRule: sender]; [self copyRule: self]; return self;}

- enterTRule:sender {char ch[WNML];
char    tiempo[30], fecha[30];
struct  tm  *timeptr;
time_t  secsnow;

   time(&secsnow);
   timeptr = localtime(&secsnow);
   sprintf(tiempo,"%02d:%02d:%02d\n",
	timeptr->tm_hour, 
	timeptr->tm_min,
        timeptr->tm_sec);
   [tiTime setStringValue: tiempo];
   sprintf(fecha,"%d-%d-19%02d\n",
	((timeptr->tm_mon) + 1), 
	timeptr->tm_mday,
        timeptr->tm_year);
   [tiDat setStringValue: fecha];

     ttrule[[sender selectedCol]] = [[sender selectedCell] state]+'0';
     totoasc();
     wrru(ch,ascrule);
     [ruleNumber setStringValue: ch];
     [rulePanel setAllRules: self];
     [SqarWindow setDocEdited: NO];
     if (istot()) [tRuleNumber setIntValue: totonu(0)];
       else [tRuleNumber setStringValue: "enter totalistic #"];
     [bernView drawBern: self];
return self;
}

- copyTRule:sender {char ch[WNML]; int i;
     wrru(ch,ascrule);
     [ruleNumber setStringValue: ch];
     if (istot()) {
		[tRuleNumber setIntValue: totonu(0)];
		for (i=0; i<TS; i++) 
		  [[theToMatrix cellAt: 0: i] setIntValue: ttrule[i]-'0'];
		}
       else [tRuleNumber setStringValue: "enter totalistic #"];
     [SqarWindow setDocEdited: NO];
     [bernView selfBern: nil];
return self;
}

/* -------------------------------- * * * * -------------------------------- */

- initFrame:(const NXRect *)frameRect
/*
 * Initializes the new VN4DView object.  First, an initFrame: message is sent 
 * to super to initialize VN4DView as a View.  Next, the VN4DView sets its own 
 * state -- that it is opaque and that the origin of its coordinate system lies 
 * in the center of its area.  It then creates and initializes its associated 
 * objects, a Storage object, an NXCursor, and a Cell.  Finally, it loads into 
 * the Window Server some PostScript procedures that it will use in drawing 
 * itself. 
 */
{
    NXPoint	spot;
	
    [super initFrame:frameRect];
/*  [self translate:floor(frame.size.width/2) :floor(frame.size.height/2)]; */
    [self setOpaque:YES];
    crossCursor=[NXCursor newFromImage:[NXImage newFromSection:"cross.tiff"]];
    spot.x = spot.y = 7.0;
    [crossCursor setHotSpot:&spot];

  PSsetgray(NX_BLACK);
  PSselectfont("Times-Roman",16.0);

  [self setFlipped: YES];
  drawview=7;			      /* prefer origin in upper left corner */
	return self;
}

- setDelegate:anObject
/*
 * Sets the VN4DView's delegate instance variable to the supplied object.
 */
{
	delegate = anObject;
	return self;
}

- drawSelf:(const NXRect *)rects :(int)rectCount
/*
 * Draws the VN4DView's background and axes.  If there are any points,
 * these are drawn too.
 *
 * (Note:  For simplicity, although VN4DView only repaints the background 
 * of the update region, it redraws the entire axes.  For better performance, 
 * only those parts of the axes that fall within the update region should 
 * be redrawn.)   
 */
{
	int		 i, j, k, l, n, ii, jj;
        double           p, q, r, s, t, mo;
	NXRect		 plotRect;

	if (rects == NULL) return self;

	/* paint visible area white then draw axes */
	PSsetgray(NX_WHITE);
	if (drawARBack) NXRectFill(&rects[0]);
	PSsetgray(NX_DKGRAY);

	if (drawBnstein==YES) {
// 	  for (i=0; i<TS; i++) bp[i]=0.0;
//  	  for (i=0; i<NG; i++) for (j=0; j<MB; j++) {
//  		if ([[theZMatrix cellAt: j: i] state]) bp[gtb[i]]+=1.0;
// 		if ([[theOMatrix cellAt: j: i] state]) bp[gtb[i]+1]+=1.0;
//  		}
	berncoeff();
	  argfram(50,3);
	  argdiag(50,3);
	  bargraf(TS-1,50);
	  }

	if (drawTrajec==YES) {
	  argfram(75,3);
	  argdiag(75,3);
	  bargraf(TS-1,75);
	  statecount(arr1,8,8,8,8);
	  for (j=0; j<KK; j++) llf[j]=lf[j];
	  aronegen(arr1,brr2,8,8,8,8);
	  if (pDots) arbdot((75*llf[1])/4096,(75*lf[1])/4096,3);
	    else arbline(0.0367*((double)lf[1]),0.0367*((double)llf[1]),0);
	  for (i=0; i<trajLen; i++) {
	    for (j=0; j<KK; j++) llf[j]=lf[j];
	    aronegen(arr1,brr2,8,8,8,8);
	    if (pDots) arbdot((75*llf[1])/4096,(75*lf[1])/4096,3);
	  else arbline(0.0367*((double)lf[1]),0.0367*((double)llf[1]),1);
	    }
	  PSstroke();
	  }

	if (drawRturn==YES) {
	  argfram(75,3);
	  argdiag(75,3);
	  bargraf(TS-1,75);
	  for (i=0; i<75; i++) {
	    p=((double)i)/75.0; q=1.0-p;
	    ranblin(q,p);
	    for(j=0; j<rturnNum; j++) aronegen(arr1,brr2,8,8,8,8);
	    arbdot(i,(75*lf[1])/4096,3);
	    }
	  }

	if (drawFreqs==YES) {
	  argfram(75,3);
	  for (i=0; i<75; i++) {
	    aronegen(arr1,brr2,8,8,8,8);
	    for (j=0, k=0; j<TS; j++)
		if (tus[j]>57) for (l=0; l<tus[j]; l+=58) arbdot(i,k++,j%3);
	    arbdot(i,(75*lf[1])/4096,3);
	    }
printf("teecees:");
for (i=0; i<DS; i++) printf("%5d ",tc[i]);
printf("\n");
printf("tussies: ");
for (i=0; i<TS; i++) printf("%5d ",tus[i]);
printf("\n");
	  }

	if (slowFreqs==YES) {
	  argfram(75,3);
	  for (i=0; i<75; i+=3) {
	    aronegen(arr1,brr2,8,8,8,8);
	    for (j=0, k=0; j<TS; j++)
		if (tus[j]>57) for (l=0; l<tus[j]; l+=58) {
		  arbdot(i,k,j%3);
		  arbdot(i+1,k,j%3);
		  arbdot(i+2,k,j%3);
		  k++;
		  }
	    arbdot(i,(75*lf[1])/4096,3);
	    arbdot(i+1,(75*lf[1])/4096,3);
	    arbdot(i+2,(75*lf[1])/4096,3);
	    }
	  }

	if (drawQuency==YES) {
printf("drawing Quency\n");
	  if (stretch==125) argfram(125,3);
	  n=stretch/3+2;
	  for (i=0; i<stretch; i++) {
	    ii=i; if (skipThree) ii=i/3+n*(i%3);
		aronegen(arr1,brr2,8,8,8,8);
	    for (j=0, k=0; j<TS; j++) {
		jj=totpmut[j];
		NXSetColor(totcolst[jj]);
		if (tus[jj]>32) 
		    for (l=0; l<tus[jj]; l+=33) arbdot(ii,k++,5);
		}
	    arbdot(ii,(125*lf[1])/4096,3);
	    }
printf("totpmut: ");
for (i=0; i<TS; i++) printf("%5d ",totpmut[i]);
printf("\n");
printf("tussies: ");
for (i=0; i<TS; i++) printf("%5d ",tus[i]);
printf("\n");
	  }

        if (checkBoard==YES) {
   	  [self getBounds: &plotRect];
	  mo=(double)moTiles;
	  j=(int)(plotRect.size.width/mo);
	  i=(int)(plotRect.size.height/mo);
	  zerline();
          for (k=0; k<i/2; k++) {
	    for (l=0; l<j/2; l++) arr1[k*j+l]=0;
            for (l=j/2; l<j; l++) arr1[k*j+l]=1;
            }
          for (k=i/2; k<i; k++) {
	    for (l=0; l<j/2; l++) arr1[k*j+l]=1;
            for (l=j/2; l<j; l++) arr1[k*j+l]=0;
            }
	  }

        if (smallSmall==YES) {
	  mo=(double)moTiles;
   	  [self getBounds: &plotRect];
	  j=(int)(plotRect.size.width/mo);
	  i=(int)(plotRect.size.height/mo);
	  zerline();
          for (k=i/3; k<2*(i/3); k++) for (l=j/3; l<2*(j/3); l++) 
		arr1[k*j+l]=random()%KK;
	  }

        if (drawSmall==YES) {
	  [window setTitle: "evolution"];
   	  [self getBounds: &plotRect];
	  j=(int)(plotRect.size.width/4.0);       /* 4.0 for drawview=7 */
	  i=(int)(plotRect.size.height/4.0);
	  if (AL > i*j) {
	    ii=(int)sqrt((double)(AL/(i*j)));
	    arscreen(arr1,i,j,ii,ii,0,0);
	    if(repPlEv>0) aronegen(arr1,brr2,i,j,ii,ii);
	    } else printf("small volume = %6d\n",j*i*i);
	  }

        if (smallMosaic==YES) {
	  mo=(double)moTiles;
	  [window setTitle: "mosaic"];
   	  [self getBounds:&plotRect];
	  j=(int)(plotRect.size.width/mo);
	  i=(int)(plotRect.size.height/mo);
	  if (AL > i*j) {
	    ii=(int)sqrt((double)(AL/(i*j)));
	    for (k=0; k<moTiles; k++) for (l=0; l<moTiles; l++) {
	      manyrect(k,l,i,j,ii,ii,0,0);
	      [window flushWindow];
	      }
	    smallgrid(i,j);
	    } else printf("volume = %6d\n",i*j*j);
	  }

        if (drawVNPlane==YES) {}

        if (drawVNMosaic==YES) {}

	return self;
}

- sizeTo:(NXCoord)width :(NXCoord)height
/*
 * Ensures that whenever the VN4DView is resized, the origin of its 
 * coordinate system is repositioned to the center of its area.
 */
{
//	  [self setFlipped: YES];
	[super sizeTo: width: height];
	if (drawview==4||drawview==6||drawview==7||drawview>=10) 
		{[self setDrawOrigin:0 :0]; return self;}
	else
		[self setDrawOrigin: -floor(width/2) : -floor(height/2)]; 
	return self;
}

- mouseDown:(NXEvent *) theEvent
/*
 * Responds to a message the system sends whenever the user presses the mouse
 * button when the cursor is over the VN4DView.  The VN4DView changes the 
 * cursor to a cross-hairs image and then starts asking for mouse-dragged or 
 * mouse-up events.  As it receives mouse-dragged events, the VN4DView updates 
 * the readOut text Cell with the cursor's coordinates.  If the user releases 
 * the mouse button while the cursor is over the VN4DView, the VN4DView 
 * registers the point and then sends a message to its delegate notifying it 
 * of the new point.
 */
{
	int	looping = YES, oldMask;
	NXPoint	aPoint;
	NXRect	plotRect;

      /*   double  x, y,  eks, wye; */
	 double ex, wy;
	 int k;

	[crossCursor set];
	oldMask = [window addToEventMask:NX_MOUSEDRAGGEDMASK];
 				[window setDocEdited:YES];
		                aPoint = theEvent->location;
				aPoint.y = aPoint.y;
		                [self convertPoint:&aPoint fromView:nil];
                                ex=aPoint.x;
                                wy=aPoint.y;
				k=nearest(ex,wy);

//       if (theEvent->data.mouse.click==3) {} else 
//       if (theEvent->data.mouse.click==2) {} else 
	{
	[self lockFocus];
//         PSsetinstance(YES);
	do {
// 		PSnewinstance();
		aPoint = theEvent->location;
		[self convertPoint:&aPoint fromView:nil];
		aPoint.y= (plotRect.size.height-1)-aPoint.y;
                ex=aPoint.x;
                wy=aPoint.y;
				PSsetgray(NX_BLACK);
		[window flushWindow];
		if (theEvent->type == NX_MOUSEUP) {
	/* on mouse-up, register point, inform delegate, and clean up state */
			[window flushWindow];
			looping = NO;
		}	
   	} while (looping && (theEvent = 
		[NXApp getNextEvent:NX_MOUSEUPMASK|NX_MOUSEDRAGGEDMASK]));
	[self unlockFocus];

	}

	[window setEventMask:oldMask];
	[NXArrow set];
	return self;
}

@end

/*  - - - - - - - -  n x s q a r 2 1 . c  - - - - - - - - - - - - - - - - */

/* SQAR21.C					*/
/* Four-Dimensional Square Lattice Life		*/
/* binary cells, first neighbors		*/
/* ---- von Neumann neighborhoods ----		*/
/*						*/
/* Reference:					*/
/*						*/
/* Harold V. McIntosh, 16 May 1987		*/
/* 7 July 1987 - separate rules for each state	*/
/* 8 July 1988 - binary, 3 neighborhood classes	*/
/* 9 July 1988 - hexagonal lattice 		*/
/* 20 July 1988 - von Neumann neighborhgood	*/

void asctoqar() {int i0, i1, i2, i3, i4, i5, i6, i7, i8;
  for (i0=0; i0<KK; i0++)
  for (i1=0; i1<KK; i1++)
  for (i2=0; i2<KK; i2++)
  for (i3=0; i3<KK; i3++)
  for (i4=0; i4<KK; i4++)
  for (i5=0; i5<KK; i5++)
  for (i6=0; i6<KK; i6++)
  for (i7=0; i7<KK; i7++)
  for (i8=0; i8<KK; i8++)
    sqrule[i0+KK*(i1+KK*(i2+KK*(i3+KK*(i4+KK*(i5+KK*(i6+KK*(i7+KK*i8)))))))] = 
	ascrule[i0][i1][i2][i3][i4][i5][i6][i7][i8];
}

void qartoasc() {int i0, i1, i2, i3, i4, i5, i6, i7, i8;
  for (i0=0; i0<KK; i0++)
  for (i1=0; i1<KK; i1++)
  for (i2=0; i2<KK; i2++)
  for (i3=0; i3<KK; i3++)
  for (i4=0; i4<KK; i4++)
  for (i5=0; i5<KK; i5++)
  for (i6=0; i6<KK; i6++)
  for (i7=0; i7<KK; i7++)
  for (i8=0; i8<KK; i8++)
    ascrule[i0][i1][i2][i3][i4][i5][i6][i7][i8] = 
      sqrule[i0+KK*(i1+KK*(i2+KK*(i3+KK*(i4+KK*(i5+KK*(i6+KK*(i7+KK*i8)))))))];
}

/* calculate one generation of evolution */
/* aa[m][n] = original and final plane   */
/* bb[m][n] = intermediate scratch plane */
/* m = number of rows on the torus       */
/* n = number of columns on the torus    */
void aronegen(ar,br,k,l,m,n) char *ar; int *br; int k, l, m, n; {
int i0, i1, i2, i3, gn;

/* count neighbors of center hypercube */

for (i0=0; i0<k; i0++) for (i1=0; i1<l; i1++) 
for (i2=0; i2<m; i2++) for (i3=0; i3<n; i3++) {
	    gn=      ar[inx4(i0-1,i1,i2,i3,k,l,m,n)];
	    gn=KK*gn+ar[inx4(i0+1,i1,i2,i3,k,l,m,n)];
	    gn=KK*gn+ar[inx4(i0,i1-1,i2,i3,k,l,m,n)];
	    gn=KK*gn+ar[inx4(i0,i1+1,i2,i3,k,l,m,n)];
	    gn=KK*gn+ar[inx4(i0,i1,i2-1,i3,k,l,m,n)];
	    gn=KK*gn+ar[inx4(i0,i1,i2+1,i3,k,l,m,n)];
	    gn=KK*gn+ar[inx4(i0,i1,i2,i3-1,k,l,m,n)];
	    gn=KK*gn+ar[inx4(i0,i1,i2,i3+1,k,l,m,n)];
	    gn+=  GH*ar[inx4(i0,i1,i2,i3,k,l,m,n)];
  br[inx4(i0,i1,i2,i3,k,l,m,n)]=gn;
  }

/* phase to update cells */
for (i0=0; i0<KK; i0++)   lf[i0] =0;
for (i0=0; i0<TS; i0++)   tus[i0]=0;
for (i0=0; i0<DS; i0++)   rus[i0]=0;
for (i0=0; i0<2*NG; i0++) gus[i0]=0;
for (i0=0; i0<k; i0++) for (i1=0; i1<l; i1++) 
for (i2=0; i2<m; i2++) for (i3=0; i3<n; i3++) {
  gn=br[inx4(i0,i1,i2,i3,k,l,m,n)];
  ar[inx4(i0,i1,i2,i3,k,l,m,n)]=sqrule[gn]-'0';
  lf[ar[inx4(i0,i1,i2,i3,k,l,m,n)]]+=1;
  gus[gotb[gn]]+=1;
  tus[tc[gn]]+=1;
  rus[gn]++;
  }

/* phase to show results */
}	 /* end aronegen */
    
/*find fraction of live cells in arry[m][n] */
void statecount(aa,k,l,m,n) char *aa; int k, l, m, n; {int i0, i1, i2, i3;
  for (i0=0; i0<KK; i0++) lf[i0]=0;
  for (i0=0; i0<k; i0++) for (i1=0; i1<l; i1++) 
  for (i2=0; i2<m; i2++) for (i3=0; i3<n; i3++) 
	lf[aa[inx4(i0,i1,i2,i3,k,l,m,n)]]+=1;
}

int inx4(i0,i1,i2,i3,k,l,m,n) int i0, i1, i2, i3, k, l, m, n; {
  return((k+i0)%k+k*((l+i1)%l+l*((m+i2)%m+m*((n+i3)%n)))); }

/* display a screen of evolution */
void arscreen(aa,k,l,m,n,p,q) char *aa; int k, l, m, n, p, q; {int i, j;
  for (j=0; j<k; j++) for (i=0; i<l; i++) 
	videodot(j,i,aa[i+k*(j+l*((p%m)+m*(q%n)))]);}

/* make a frame for the fixpoint graph  */
/* n     = dimension of arframe         */
void argfram(n,l) int n, l; {int i;
for (i=0; i<=n; i++) 
	{arbdot(i,0,0);arbdot(i,n,0);arbdot(n,i,0);arbdot(0,i,0);};
for (i=0; i<=2*n; i+=15) 
	{arbdot(i/2,0,l);arbdot(i/2,n,l);arbdot(n/2,i,l);arbdot(0,i/2,l);};
}

/* put a diagonal in the fixpoint graph */
void argdiag(n,l) int n, l; {int i; for (i=0; 2*i<n; i++) arbdot(2*i,2*i,l);}

/* graph Bernstein polynomial */
void bargraf(k,n) int k, n; {int i; double arburn(), en, p;
en=(double)(n);
for (i=0; i<n; i++) {
  p=((double)(i))/en;
  arbdot(i,(int)(en*arburn(p,k)),1);
  }
}

/* evaluate the nth degree Bernstein polynomial at point p */
double arburn(p,n) double p; int n; {double q, s, x, r; int i;
if (p>0.99) return bp[n];
q=1.0-p; r=p/q; s=0.0; x=1.0;
for (i=0; i<n;  i++) x*=q;
for (i=0; i<=n; i++, x*=r) s+=bp[i]*x;
s*=0.9998;
return(s+0.0001);
}

/* arbdot(i,j,l) - place dot on Bernstein screen at (i,j) in color l */
void arbdot(i,j,l) int i, j, l; {videodot(j,i,l);}

/* arbline(x,y,l) - place line on Bernstein screen to (x,y); l=first */
void arbline(x,y,l) double x, y; int l; 
	{if (l==0) PSmoveto(y,150.0-x); else PSlineto(y,150.0-x);}

/* display Wolfram rule number of planar cross section */
/* calculate LCAU21 wru from a SQUL21 argument rule    */
void warsscrrul(c,ru) char *c, ru[KK][KK][KK][KK][KK][KK][KK][KK][KK]; {
char ur[KK][KK][KK]; int i0, i1, i2;
  for (i0=0; i0<KK; i0++) for (i1=0; i1<KK; i1++) for (i2=0; i2<KK; i2++)
    ur[i0][i1][i2]=ru[i0][i1][i1][i1][i1][i1][i1][i2][i1];
  wrru21(c,ur);
  }

// /* calculate wru from an argument rule	*/
// void wrru21(c,ru) char *c, ru[KK][KK][KK]; {int  k, i0, i1, i2;
//   k=0;
//   for (i0=KK-1; i0>=0; i0--)
//   for (i1=KK-1; i1>=0; i1--)
//   for (i2=KK-1; i2>=0; i2--)
//     k=2*k+(ru[i0][i1][i2]-'0');
//   sprintf(c,"%3d",k);
// }

/* end VN4DView.m */
